###############################################################################
## Company      : 4JS
## Program      : schema.4gl
## Date         : 19/07/1995
## Version      : 1.00
##
## Title        : Generation of schema for Four J's Universal Compiler
## Last Changes : 03/06/97
## version      : 1.10.1a
## Update       : Accept synonym
## Last Changes : 15/04/98
## version      : 1.20.1a
## Update       : Ignore new type of Informix Universal Database
###############################################################################
## Version 2.03.2b
## error in SQL colno replaces by coltype
## Version 2.03.2a
## $Id: schema.4gl,v 1.5 1998/09/30 22:47:22 jleffler Exp $
## $Log: schema.4gl,v $
## Revision 1.5  1998/09/30 22:47:22  jleffler
## Merge in the 2.03.1a version of schema, which already
## contains a remote database fix, so delete most of the last
## fix.  Update the code which tests for the UPSCOL tables to
## work in MODE ANSI databases.
##
## Revision 1.4  1998/09/30 21:30:48  jleffler
## The compilers (fgl2p, fgl2c) ignore the @server part
## of database@server, so fglschema should too.
##
## Revision 1.3  1998/09/03 20:32:42  jleffler
## Informix-SE does not support either remote synonyms
## in general or the keyword DBSERVERNAME.  Consequently, the
## code which tests for remote synonyms and uses DBSERVERNAME
## should only be executed if there are any remote synonyms
## to test.
##
## Revision 1.2  1998/07/09 19:28:01  jleffler
## version      : 2.02.1a
## Systematically use 'informix'.systables in place of systables
## so that the code will work in most MODE ANSI databases (where the
## tablenames are unique -- when the same tablename exists with two
## different owners, there will be problems!).
## Add code to handle synonyms referencing tables in another database.
## Note that this uses the MOD() function which is available in 7.2x.
## It should be present in 5.x and 6.x databases too, but this has
## not been proved.  If it is, then the same clause could be used in
## the main (local tables, local synonyms) search code and the data
## does not have to be filtered to the fglschema program and back into
## the database again -- which should be more efficient.
##
##
## version      : 2.03.1a ( Fri Jul 10 12:12:34 MET DST 1998 )
## syntax is accepted for every engine since version 4.x
## accepts to create schema of remote database
###############################################################################

MAIN
  DEFINE dbname     CHAR(128)
  DEFINE fn         CHAR(256)
  DEFINE msg        CHAR(80)
  DEFINE fname      CHAR(64)
  DEFINE i          SMALLINT
  DEFINE r          RECORD
                    tabname   CHAR(18),
                    colname   CHAR(18),
                    coltype   SMALLINT,
                    collength SMALLINT,
                    colno     SMALLINT
                    END RECORD
  DEFINE rsyn       RECORD
                    ltabname    CHAR(18),
                    rservername CHAR(18),
                    dbname      CHAR(18),
                    owner       CHAR(8),
                    rtabname    CHAR(18),
                    lservername CHAR(18)
                    END RECORD
  DEFINE stmt       CHAR(400)
  DEFINE n_remote   INTEGER

  IF num_args() = 1 THEN
    IF UPSHIFT(arg_val(1)) = "-V" THEN
      DISPLAY "Schema for 4GL Compiler Version 2.03.2b"
      EXIT PROGRAM 0
    END IF
    LET dbname = arg_val(1)
  ELSE
    LET msg = "Usage: ", arg_val(0), " dbname"
    DISPLAY msg CLIPPED
    EXIT PROGRAM
  END IF

  DATABASE dbname

  IF status THEN
    LET msg = "Error:", err_get(status)
    DISPLAY msg
    EXIT PROGRAM
  END IF

  CREATE TEMP TABLE myschema (
      tabname   CHAR(18),
      colname   CHAR(18),
      coltype   SMALLINT,
      collength SMALLINT,
      colno     SMALLINT
  )

  DECLARE databaseschema CURSOR FOR
    SELECT t.tabname, c.colname, coltype , collength, colno
      FROM 'informix'.systables t, 'informix'.syscolumns c 
      WHERE t.tabid = c.tabid 
    UNION
# Local synonyms are used like tables
      SELECT t.tabname, c.colname, coltype , collength, colno
        FROM 'informix'.syscolumns c,
             'informix'.syssyntable s,
             'informix'.systables t
        WHERE s.btabid = c.tabid
          AND t.tabid = s.tabid
          AND s.btabid IS NOT NULL
        ORDER BY 1, 5

  FOREACH databaseschema INTO r.*
    IF (r.coltype MOD 256) <= 16 THEN
      INSERT INTO myschema VALUES(r.*)
    END IF
  END FOREACH

# Remote synonyms are a pain...
# Furthermore, they only exist in OnLine engines, and only OnLine
# supports the DBSERVERNAME built-in function.  So, only try to deal
# with remote synonyms if there are any to deal with.

  SELECT COUNT(*)
    INTO n_remote
    FROM 'informix'.syssyntable
    WHERE btabid IS NULL

  IF n_remote > 0 THEN

    DECLARE c_remsyn CURSOR FOR
      SELECT t.tabname ltabname, s.servername rservername,
             s.dbname, s.owner, s.tabname rtabname, DBSERVERNAME lservername
        FROM 'informix'.systables t, 'informix'.syssyntable s
        WHERE t.tabid = s.tabid AND s.btabid IS NULL

    FOREACH c_remsyn INTO rsyn.*
      IF rsyn.rservername IS NULL OR rsyn.rservername = ' ' THEN
        LET rsyn.rservername = rsyn.lservername
      END IF
      LET stmt =
             "INSERT INTO MySchema ",
             "SELECT '", rsyn.ltabname, "', c.colname, c.coltype, ",
             "c.collength, c.colno FROM ",
             rsyn.dbname, "@", rsyn.rservername, ":'informix'.systables T,",
             rsyn.dbname, "@", rsyn.rservername, ":'informix'.syscolumns C ",
             "WHERE C.Tabid = T.Tabid AND T.Tabname = '", rsyn.rtabname,
             "' AND T.Owner = '", rsyn.owner CLIPPED, "'",
             "AND MOD(C.Coltype, 256) < 16"
      PREPARE p_stmt FROM stmt
      EXECUTE p_stmt
      FREE p_stmt
    END FOREACH

  END IF

  IF dbname MATCHES "*@*" AND dbname[1,1] != "@" THEN
    FOR i = 1 TO LENGTH(dbname)
      IF dbname[i,i] = "@" THEN
        LET fname = dbname[1,i-1]
        EXIT FOR
      END IF
    END FOR
  ELSE
    LET fname = dbname
  END IF

  LET fn = fname CLIPPED, ".sch"

  UNLOAD TO fn DELIMITER "^" 
      SELECT *
        FROM myschema
      ORDER BY tabname, colno

  DISPLAY "File \"", fn clipped, "\" created"

  -- UPSCOL tables: SysColVal (validation)
  SELECT *
    FROM 'informix'.systables
      WHERE tabname = "syscolval"

  IF status != NOTFOUND THEN
    LET fn = fname CLIPPED, ".val"
    UNLOAD TO fn DELIMITER "^"
      SELECT tabname, colname, attrname, attrval
        FROM syscolval
        ORDER BY tabname, colname
    DISPLAY "File \"", fn clipped, "\" created"
  END IF

  -- UPSCOL tables: SysColAtt (display attributes)
  SELECT *
    FROM 'informix'.systables
      WHERE tabname = "syscolatt"

  IF status != NOTFOUND THEN
    LET fn = fname CLIPPED, ".att"
    UNLOAD TO fn DELIMITER "^" 
      SELECT tabname, colname, seqno, color, inverse,
             underline, blink, left, def_format, condition
		FROM syscolatt
		ORDER BY tabname, colname, seqno
    DISPLAY "File \"", fn clipped, "\" created"
  END IF

END MAIN
